package gregapi.tileentity;

import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnToolClick;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_SyncDataCovers;
import gregapi.code.ArrayListNoNulls;
import gregapi.network.INetworkHandler;
import gregapi.network.IPacket;
import gregapi.network.packets.covers.PacketSyncDataIDsAndCovers;
import gregapi.network.packets.ids.PacketSyncDataIDs;
import gregapi.util.UT;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.init.Items;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.util.ForgeDirection;

/**
 * @author Gregorius Techneticies
 */
public abstract class TileEntityBase6 extends TileEntityBase5 implements IMTE_SyncDataCovers, IMTE_OnToolClick {
	protected short[] mCoverIDs, mCoverMetas, mCoverVisuals;
	protected NBTTagCompound[] mCoverNBTs;
	
	@Override
	public IPacket getClientDataPacket(boolean aSendAll) {
		return aSendAll ? mCoverIDs == null ? new PacketSyncDataIDs(xCoord, yCoord, zCoord, getMultiTileEntityRegistryID(), getMultiTileEntityID()) : new PacketSyncDataIDsAndCovers(xCoord, yCoord, zCoord, getMultiTileEntityRegistryID(), getMultiTileEntityID(), mCoverIDs, mCoverMetas, mCoverVisuals) : null;
	}
	
    @Override
	public void readFromNBT2(NBTTagCompound aNBT) {
    	super.readFromNBT2(aNBT);
    	if (aNBT.hasKey("gt.covers")) {
    		NBTTagCompound tNBT = aNBT.getCompoundTag("gt.covers");
    		mCoverIDs		= new short[] {tNBT.getShort("a"), tNBT.getShort("b"), tNBT.getShort("c"), tNBT.getShort("d"), tNBT.getShort("e"), tNBT.getShort("f")};
    		mCoverMetas		= new short[] {tNBT.getShort("g"), tNBT.getShort("h"), tNBT.getShort("i"), tNBT.getShort("j"), tNBT.getShort("k"), tNBT.getShort("l")};
    		mCoverVisuals	= new short[] {tNBT.getShort("m"), tNBT.getShort("n"), tNBT.getShort("o"), tNBT.getShort("p"), tNBT.getShort("q"), tNBT.getShort("r")};
    		mCoverNBTs		= new NBTTagCompound[] {tNBT.getCompoundTag("s"), tNBT.getCompoundTag("t"), tNBT.getCompoundTag("u"), tNBT.getCompoundTag("v"), tNBT.getCompoundTag("w"), tNBT.getCompoundTag("x")};
        }
    }
    
    @Override
	public void writeToNBT2(NBTTagCompound aNBT) {
    	super.writeToNBT2(aNBT);
    	if (mCoverIDs != null) {
    		NBTTagCompound tNBT = new NBTTagCompound();
    		tNBT.setShort("a", mCoverIDs		[0]); tNBT.setShort("b", mCoverIDs		[1]); tNBT.setShort("c", mCoverIDs		[2]); tNBT.setShort("d", mCoverIDs		[3]); tNBT.setShort("e", mCoverIDs		[4]); tNBT.setShort("f", mCoverIDs		[5]); 
    		tNBT.setShort("g", mCoverMetas		[0]); tNBT.setShort("h", mCoverMetas	[1]); tNBT.setShort("i", mCoverMetas	[2]); tNBT.setShort("j", mCoverMetas	[3]); tNBT.setShort("k", mCoverMetas	[4]); tNBT.setShort("l", mCoverMetas	[5]);
    		tNBT.setShort("m", mCoverVisuals	[0]); tNBT.setShort("n", mCoverVisuals	[1]); tNBT.setShort("o", mCoverVisuals	[2]); tNBT.setShort("p", mCoverVisuals	[3]); tNBT.setShort("q", mCoverVisuals	[4]); tNBT.setShort("r", mCoverVisuals	[5]);
    		tNBT.setTag  ("s", mCoverNBTs		[0]); tNBT.setTag  ("t", mCoverNBTs		[1]); tNBT.setTag  ("u", mCoverNBTs		[2]); tNBT.setTag  ("v", mCoverNBTs		[3]); tNBT.setTag  ("w", mCoverNBTs		[4]); tNBT.setTag  ("x", mCoverNBTs		[5]);
    		aNBT.setTag("gt.covers", tNBT);
    	}
    }
    
    @Override
	public void receiveDataCovers(short[] aCoverVisuals, INetworkHandler aNetworkHandler) {
		mCoverVisuals = aCoverVisuals;
	}
    
    @Override
	public void receiveDataCovers(short[] aCoverIDs, short[] aCoverMetas, INetworkHandler aNetworkHandler) {
		mCoverIDs = aCoverIDs;
		mCoverMetas = aCoverMetas;
	}
    
    @Override
    public ArrayListNoNulls<ItemStack> getDrops(int aFortune, boolean aSilkTouch) {
    	ArrayListNoNulls<ItemStack> rList = super.getDrops(aFortune, aSilkTouch);
    	if (mCoverIDs != null && rList.size() == 1) {
    		NBTTagCompound tNBT = new NBTTagCompound(), aNBT = rList.get(0).getTagCompound();
    		tNBT.setShort("a", mCoverIDs		[0]); tNBT.setShort("b", mCoverIDs		[1]); tNBT.setShort("c", mCoverIDs		[2]); tNBT.setShort("d", mCoverIDs		[3]); tNBT.setShort("e", mCoverIDs		[4]); tNBT.setShort("f", mCoverIDs		[5]); 
    		tNBT.setShort("g", mCoverMetas		[0]); tNBT.setShort("h", mCoverMetas	[1]); tNBT.setShort("i", mCoverMetas	[2]); tNBT.setShort("j", mCoverMetas	[3]); tNBT.setShort("k", mCoverMetas	[4]); tNBT.setShort("l", mCoverMetas	[5]);
    		tNBT.setTag  ("s", mCoverNBTs		[0]); tNBT.setTag  ("t", mCoverNBTs		[1]); tNBT.setTag  ("u", mCoverNBTs		[2]); tNBT.setTag  ("v", mCoverNBTs		[3]); tNBT.setTag  ("w", mCoverNBTs		[4]); tNBT.setTag  ("x", mCoverNBTs		[5]);
    		if (aNBT == null) aNBT = new NBTTagCompound();
    		aNBT.setTag("gt.covers", tNBT);
    		rList.get(0).setTagCompound(aNBT);
    	}
    	return rList;
    }
    
	@Override
	public long onToolClick(String aTool, long aRemainingDurability, Entity aPlayer, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, byte aSide, float aHitX, float aHitY, float aHitZ) {
		if (worldObj.isRemote) return 0;
		if (aTool.equals("crowbar") && mCoverIDs != null && mCoverIDs[aSide] != 0) {
			ItemStack tStack = getCoverItem(aSide);
			setCoverItem(aSide, null, aPlayer, false, true);
			worldObj.spawnEntityInWorld(new EntityItem(worldObj, xCoord+0.5+ForgeDirection.getOrientation(aSide).offsetX, yCoord+0.5+ForgeDirection.getOrientation(aSide).offsetY, zCoord+0.5+ForgeDirection.getOrientation(aSide).offsetZ, tStack));
			UT.Sounds.send(worldObj, "random.break", 1.0F, -1.0F, xCoord, yCoord, zCoord);
			return 10000;
		}
		return 0;
	}
	
	public ItemStack getCoverItem(byte aSide) {
		return mCoverIDs == null && mCoverIDs[aSide] == 0 ? null : UT.Stacks.create(mCoverIDs[aSide], 1, mCoverIDs[aSide], mCoverNBTs[aSide]);
	}
	
	public boolean setCoverItem(byte aSide, ItemStack aStack, Entity aPlayer, boolean aForce, boolean aBlockUpdate) {
		updateClientData();
		boolean rReturn = true;
		
		if (mCoverIDs == null) {mCoverIDs = new short[6]; mCoverMetas = new short[6]; mCoverVisuals = new short[6]; mCoverNBTs = new NBTTagCompound[6];}
		
		mCoverIDs[aSide] = aStack == null ? 0 : (short)Item.getIdFromItem(aStack.getItem());
		mCoverMetas[aSide] = aStack == null ? 0 : (short)Items.feather.getDamage(aStack);
		mCoverNBTs[aSide] = aStack == null ? null : aStack.getTagCompound();
		
		if (aBlockUpdate) causeBlockUpdate();
		
		for (byte i = 0; i < 6; i++) if (mCoverIDs[i] != 0) return rReturn;
		mCoverIDs = null; mCoverMetas = null; mCoverVisuals = null; mCoverNBTs = null;
		return rReturn;
	}
}